@@ -32,7 +32,7 @@ func newBuildsCommand() *cobra.Command {
 	cmd.AddCommand(newBuildsCancelCommand())
 	cmd.AddCommand(newBuildsShowCommand())
 	cmd.AddCommand(newBuildsListCommand())
-	cmd.AddCommand(newBuildsSecretsCommand())
+	cmd.AddCommand(newBuildsSecretCommand())
 	cmd.AddCommand(newBuildsSSHCommand())
 	cmd.AddCommand(newBuildsArtifactsCommand())
 	cmd.AddCommand(newBuildsUserWebhookCommand())
@@ -48,11 +48,16 @@ const buildsSubmitPrefill = `
 
 func newBuildsSubmitCommand() *cobra.Command {
 	var follow, edit bool
-	var note, tagString string
+	var note, tagString, visibility string
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("builds", cmd)
 
+		buildsVisibility, err := buildssrht.ParseVisibility(visibility)
+		if err != nil {
+			log.Fatal(err)
+		}
+
 		filenames := args
 		if len(args) == 0 {
 			if _, err := os.Stat(".build.yml"); err == nil {
@@ -103,21 +108,25 @@ func newBuildsSubmitCommand() *cobra.Command {
 		}
 
 		for _, manifest := range manifests {
-			job, err := buildssrht.Submit(c.Client, ctx, manifest, tags, &note)
+			job, err := buildssrht.Submit(c.Client, ctx, manifest, tags, &note, &buildsVisibility)
 			if err != nil {
 				log.Fatal(err)
 			}
 
-			fmt.Printf("%v/%v/job/%v\n", c.BaseURL, job.Owner.CanonicalName, job.Id)
+			if termfmt.IsTerminal() {
+				log.Printf("Started build %v/%v/job/%v", c.BaseURL, job.Owner.CanonicalName, job.Id)
+			} else {
+				fmt.Printf("%v/%v/job/%v\n", c.BaseURL, job.Owner.CanonicalName, job.Id)
+			}
 
 			if follow {
 				id := job.Id
-				job, err := c.followJob(ctx, job.Id)
+				job, err := followJob(ctx, c, job.Id)
 				if err != nil {
 					log.Fatal(err)
 				}
 				if job.Status != buildssrht.JobStatusSuccess {
-					c.offerSSHConnection(ctx, id)
+					offerSSHConnection(ctx, c, id)
 				}
 			}
 		}
@@ -135,12 +144,14 @@ func newBuildsSubmitCommand() *cobra.Command {
 	cmd.RegisterFlagCompletionFunc("note", cobra.NoFileCompletions)
 	cmd.Flags().StringVarP(&tagString, "tags", "t", "", "job tags (slash separated)")
 	cmd.RegisterFlagCompletionFunc("tags", cobra.NoFileCompletions)
+	cmd.Flags().StringVarP(&visibility, "visibility", "v", "unlisted", "builds visibility")
+	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
 	return cmd
 }
 
 func newBuildsResubmitCommand() *cobra.Command {
 	var follow, edit bool
-	var note string
+	var note, visibility string
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 
@@ -158,6 +169,17 @@ func newBuildsResubmitCommand() *cobra.Command {
 			log.Fatal("failed to get build manifest: invalid job ID")
 		}
 
+		var buildsVisibility buildssrht.Visibility
+		if visibility != "" {
+			var err error
+			buildsVisibility, err = buildssrht.ParseVisibility(visibility)
+			if err != nil {
+				log.Fatal(err)
+			}
+		} else {
+			buildsVisibility = oldJob.Visibility
+		}
+
 		if edit {
 			content, err := getInputWithEditor("hut*.yml", oldJob.Manifest)
 			if err != nil {
@@ -175,21 +197,25 @@ func newBuildsResubmitCommand() *cobra.Command {
 			}
 		}
 
-		job, err := buildssrht.Submit(c.Client, ctx, oldJob.Manifest, nil, &note)
+		job, err := buildssrht.Submit(c.Client, ctx, oldJob.Manifest, nil, &note, &buildsVisibility)
 		if err != nil {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("%v/%v/job/%v\n", c.BaseURL, job.Owner.CanonicalName, job.Id)
+		if termfmt.IsTerminal() {
+			log.Printf("Started build %v/%v/job/%v", c.BaseURL, job.Owner.CanonicalName, job.Id)
+		} else {
+			fmt.Printf("%v/%v/job/%v\n", c.BaseURL, job.Owner.CanonicalName, job.Id)
+		}
 
 		if follow {
 			id := job.Id
-			job, err := c.followJob(ctx, job.Id)
+			job, err := followJob(ctx, c, job.Id)
 			if err != nil {
 				log.Fatal(err)
 			}
 			if job.Status != buildssrht.JobStatusSuccess {
-				c.offerSSHConnection(ctx, id)
+				offerSSHConnection(ctx, c, id)
 			}
 		}
 	}
@@ -205,6 +231,8 @@ func newBuildsResubmitCommand() *cobra.Command {
 	cmd.Flags().BoolVarP(&edit, "edit", "e", false, "edit manifest")
 	cmd.Flags().StringVarP(&note, "note", "n", "", "short job description")
 	cmd.RegisterFlagCompletionFunc("note", cobra.NoFileCompletions)
+	cmd.Flags().StringVarP(&visibility, "visibility", "v", "", "builds visibility")
+	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
 	return cmd
 }
 
@@ -225,7 +253,7 @@ func newBuildsCancelCommand() *cobra.Command {
 				log.Fatalf("failed to cancel job %d: %v", id, err)
 			}
 
-			fmt.Printf("%d is cancelled\n", job.Id)
+			log.Printf("%d is cancelled\n", job.Id)
 		}
 	}
 
@@ -277,7 +305,7 @@ func newBuildsShowCommand() *cobra.Command {
 		)
 
 		if follow {
-			job, err = c.followJobShow(ctx, id)
+			job, err = followJobShow(ctx, c, id)
 			if err != nil {
 				log.Fatal(err)
 			}
@@ -303,23 +331,16 @@ func newBuildsShowCommand() *cobra.Command {
 		if job.Status == buildssrht.JobStatusFailed {
 			if failedTask == -1 {
 				fmt.Printf("\nSetup log:\n")
-				if err := fetchJobLogs(ctx, new(buildLog), job); err != nil {
+				if err := fetchJobLogs(ctx, c, new(buildLog), job); err != nil {
 					log.Fatalf("failed to fetch job logs: %v", err)
 				}
 			} else {
 				name := job.Tasks[failedTask].Name
 				fmt.Printf("\n%s log:\n", name)
-				if err := fetchTaskLogs(ctx, new(buildLog), job.Tasks[failedTask]); err != nil {
+				if err := fetchTaskLogs(ctx, c, new(buildLog), job.Tasks[failedTask]); err != nil {
 					log.Fatalf("failed to fetch task logs: %v", err)
 				}
 			}
-
-			cmd, err := getSSHCommand(job)
-			if err != nil {
-				log.Fatal(err)
-			}
-
-			fmt.Printf("\nSSH Command: %s\n", cmd)
 		}
 	}
 
@@ -343,10 +364,11 @@ func newBuildsListCommand() *cobra.Command {
 		if len(args) > 0 {
 			username = strings.TrimLeft(args[0], ownerPrefixes)
 		}
+
 		pagerify(func(p pager) bool {
 			var jobs *buildssrht.JobCursor
 			if len(username) > 0 {
-				user, err := buildssrht.JobsByUser(c.Client, ctx, username)
+				user, err := buildssrht.JobsByUser(c.Client, ctx, username, cursor)
 				if err != nil {
 					log.Fatal(err)
 				} else if user == nil {
@@ -355,7 +377,7 @@ func newBuildsListCommand() *cobra.Command {
 				jobs = user.Jobs
 			} else {
 				var err error
-				jobs, err = buildssrht.Jobs(c.Client, ctx, nil)
+				jobs, err = buildssrht.Jobs(c.Client, ctx, cursor)
 				if err != nil {
 					log.Fatal(err)
 				}
@@ -519,15 +541,22 @@ func newBuildsUserWebhookListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("builds", cmd)
+		var cursor *buildssrht.Cursor
 
-		webhooks, err := buildssrht.UserWebhooks(c.Client, ctx)
-		if err != nil {
-			log.Fatal(err)
-		}
+		pagerify(func(p pager) bool {
+			webhooks, err := buildssrht.UserWebhooks(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
+			}
+
+			for _, webhook := range webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = webhooks.Cursor
+			return cursor == nil
+		})
 
-		for _, webhook := range webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
 	}
 
 	cmd := &cobra.Command{
@@ -579,38 +608,59 @@ func printJob(w io.Writer, job *buildssrht.Job) {
 	}
 	fmt.Fprintln(w)
 
-	if job.Note != nil {
+	if job.Group != nil && len(job.Group.Jobs) > 1 {
+		fmt.Fprintf(w, "Group: ")
+		for _, j := range job.Group.Jobs {
+			if j.Id == job.Id {
+				continue
+			}
+			fmt.Fprintf(w, "%s %s  ", j.Status.TermIcon(), termfmt.DarkYellow.Sprintf("#%d", j.Id))
+		}
+	}
+
+	if job.Note != nil && *job.Note != "" {
 		fmt.Fprintln(w, "\n"+indent(strings.TrimSpace(*job.Note), "  "))
 	}
 
 	fmt.Fprintln(w)
 }
 
-func newBuildsSecretsCommand() *cobra.Command {
+func newBuildsSecretCommand() *cobra.Command {
+	cmd := &cobra.Command{
+		Use:   "secret",
+		Short: "Manage secrets",
+	}
+	cmd.AddCommand(newBuildsSecretListCommand())
+	cmd.AddCommand(newBuildsSecretShareCommand())
+	return cmd
+}
+
+func newBuildsSecretListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("builds", cmd)
+		var cursor *buildssrht.Cursor
 
-		secrets, err := buildssrht.Secrets(c.Client, ctx)
-		if err != nil {
-			log.Fatal(err)
-		}
-
-		tw := tabwriter.NewWriter(os.Stdout, 0, 2, 2, ' ', 0)
-		defer tw.Flush()
-		for _, secret := range secrets.Results {
-			// TODO: Display secret type (and path, mode for files)
-			created := termfmt.Dim.String(humanize.Time(secret.Created.Time))
-			s := fmt.Sprintf("%s\t%s\n", termfmt.DarkYellow.Sprint(secret.Uuid), created)
-			if secret.Name != nil {
-				s += fmt.Sprintf("%s\n", *secret.Name)
+		pagerify(func(p pager) bool {
+			secrets, err := buildssrht.Secrets(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
 			}
-			fmt.Fprintln(tw, s)
-		}
+
+			for i, secret := range secrets.Results {
+				if i != 0 {
+					fmt.Fprintln(p)
+				}
+				printSecret(p, &secret)
+			}
+
+			cursor = secrets.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
-		Use:   "secrets",
+		Use:   "list",
 		Short: "List secrets",
 		Args:  cobra.ExactArgs(0),
 		Run:   run,
@@ -618,12 +668,65 @@ func newBuildsSecretsCommand() *cobra.Command {
 	return cmd
 }
 
+func printSecret(w io.Writer, secret *buildssrht.Secret) {
+	var s string
+	created := termfmt.Dim.String(humanize.Time(secret.Created.Time))
+	s += fmt.Sprintf("%s\t%s\n", termfmt.DarkYellow.Sprint(secret.Uuid), created)
+
+	if secret.FromUser != nil {
+		s += termfmt.Dim.Sprintf("Shared by %s\n", secret.FromUser.CanonicalName)
+	}
+
+	if secret.Name != nil && *secret.Name != "" {
+		s += fmt.Sprintf("%s\n", *secret.Name)
+	}
+
+	switch v := secret.Value.(type) {
+	case *buildssrht.SecretFile:
+		s += fmt.Sprintf("File: %s %o\n", v.Path, v.Mode)
+	case *buildssrht.SSHKey:
+		s += "SSH Key\n"
+	case *buildssrht.PGPKey:
+		s += "PGP Key\n"
+	}
+
+	fmt.Fprint(w, s)
+}
+
+func newBuildsSecretShareCommand() *cobra.Command {
+	var userName string
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		c := createClient("builds", cmd)
+		userName = strings.TrimLeft(userName, ownerPrefixes)
+
+		secret, err := buildssrht.ShareSecret(c.Client, ctx, args[0], userName)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		log.Printf("Shared secret %q with user %q\n", secret.Uuid, userName)
+	}
+
+	cmd := &cobra.Command{
+		Use:               "share <secret>",
+		Short:             "Share a secret",
+		Args:              cobra.ExactArgs(1),
+		ValidArgsFunction: completeSecret,
+		Run:               run,
+	}
+	cmd.Flags().StringVarP(&userName, "user", "u", "", "username")
+	cmd.MarkFlagRequired("user")
+	cmd.RegisterFlagCompletionFunc("user", completeCoMaintainers)
+	return cmd
+}
+
 type buildLog struct {
 	offset int64
 	done   bool
 }
 
-func (c *Client) followJob(ctx context.Context, id int32) (*buildssrht.Job, error) {
+func followJob(ctx context.Context, c *Client, id int32) (*buildssrht.Job, error) {
 	ticker := time.NewTicker(time.Second)
 	defer ticker.Stop()
 
@@ -643,11 +746,11 @@ func (c *Client) followJob(ctx context.Context, id int32) (*buildssrht.Job, erro
 			}
 		}
 
-		if err := fetchJobLogs(ctx, logs[""], job); err != nil {
+		if err := fetchJobLogs(ctx, c, logs[""], job); err != nil {
 			return nil, fmt.Errorf("failed to fetch job logs: %v", err)
 		}
 		for _, task := range job.Tasks {
-			if err := fetchTaskLogs(ctx, logs[task.Name], task); err != nil {
+			if err := fetchTaskLogs(ctx, c, logs[task.Name], task); err != nil {
 				return nil, fmt.Errorf("failed to fetch task %q logs: %v", task.Name, err)
 			}
 		}
@@ -666,13 +769,13 @@ func (c *Client) followJob(ctx context.Context, id int32) (*buildssrht.Job, erro
 	}
 }
 
-func fetchJobLogs(ctx context.Context, l *buildLog, job *buildssrht.Job) error {
+func fetchJobLogs(ctx context.Context, c *Client, l *buildLog, job *buildssrht.Job) error {
 	switch job.Status {
 	case buildssrht.JobStatusPending, buildssrht.JobStatusQueued:
 		return nil
 	}
 
-	if err := fetchBuildLogs(ctx, l, job.Log.FullURL); err != nil {
+	if err := fetchBuildLogs(ctx, c, l, job.Log.FullURL); err != nil {
 		return err
 	}
 
@@ -680,13 +783,13 @@ func fetchJobLogs(ctx context.Context, l *buildLog, job *buildssrht.Job) error {
 	return nil
 }
 
-func fetchTaskLogs(ctx context.Context, l *buildLog, task buildssrht.Task) error {
+func fetchTaskLogs(ctx context.Context, c *Client, l *buildLog, task buildssrht.Task) error {
 	switch task.Status {
 	case buildssrht.TaskStatusPending, buildssrht.TaskStatusSkipped:
 		return nil
 	}
 
-	if err := fetchBuildLogs(ctx, l, task.Log.FullURL); err != nil {
+	if err := fetchBuildLogs(ctx, c, l, task.Log.FullURL); err != nil {
 		return err
 	}
 
@@ -699,7 +802,7 @@ func fetchTaskLogs(ctx context.Context, l *buildLog, task buildssrht.Task) error
 	return nil
 }
 
-func fetchBuildLogs(ctx context.Context, l *buildLog, url string) error {
+func fetchBuildLogs(ctx context.Context, c *Client, l *buildLog, url string) error {
 	if l.done {
 		return nil
 	}
@@ -711,7 +814,7 @@ func fetchBuildLogs(ctx context.Context, l *buildLog, url string) error {
 
 	req.Header.Set("Range", fmt.Sprintf("bytes=%v-", l.offset))
 
-	resp, err := http.DefaultClient.Do(req)
+	resp, err := c.HTTP.Do(req)
 	if err != nil {
 		return fmt.Errorf("HTTP request failed: %v", err)
 	}
@@ -750,16 +853,6 @@ func jobStatusDone(status buildssrht.JobStatus) bool {
 	}
 }
 
-func getSSHCommand(job *buildssrht.Job) (string, error) {
-	// TODO: compare timestamps and check if ssh access is still possible
-	if job.Runner == nil {
-		return "", errors.New("job has no runner assigned yet")
-	}
-
-	cmd := fmt.Sprintf("ssh -t builds@%s connect %d", *job.Runner, job.Id)
-	return cmd, nil
-}
-
 func parseBuildID(s string) (id int32, instance string, err error) {
 	s, _, instance = parseResourceName(s)
 	s = strings.TrimPrefix(s, "job/")
@@ -802,7 +895,7 @@ func sshConnection(job *buildssrht.Job, user string) error {
 	return cmd.Run()
 }
 
-func (c *Client) followJobShow(ctx context.Context, id int32) (*buildssrht.Job, error) {
+func followJobShow(ctx context.Context, c *Client, id int32) (*buildssrht.Job, error) {
 	ticker := time.NewTicker(time.Second)
 	defer ticker.Stop()
 
@@ -899,7 +992,7 @@ func completeBuildsUserWebhookID(cmd *cobra.Command, args []string, toComplete s
 	c := createClient("builds", cmd)
 	var webhookList []string
 
-	webhooks, err := buildssrht.UserWebhooks(c.Client, ctx)
+	webhooks, err := buildssrht.UserWebhooks(c.Client, ctx, nil)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
@@ -912,7 +1005,7 @@ func completeBuildsUserWebhookID(cmd *cobra.Command, args []string, toComplete s
 	return webhookList, cobra.ShellCompDirectiveNoFileComp
 }
 
-func (c *Client) offerSSHConnection(ctx context.Context, id int32) {
+func offerSSHConnection(ctx context.Context, c *Client, id int32) {
 	if !termfmt.IsTerminal() {
 		os.Exit(1)
 	}
@@ -934,3 +1027,24 @@ func (c *Client) offerSSHConnection(ctx context.Context, id int32) {
 		log.Fatal(err)
 	}
 }
+
+func completeSecret(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	ctx := cmd.Context()
+	c := createClient("builds", cmd)
+	var secretList []string
+
+	secrets, err := buildssrht.CompleteSecrets(c.Client, ctx)
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	for _, secret := range secrets.Results {
+		s := secret.Uuid
+		if secret.Name != nil && *secret.Name != "" {
+			s = fmt.Sprintf("%s\t%s", s, *secret.Name)
+		}
+		secretList = append(secretList, s)
+	}
+
+	return secretList, cobra.ShellCompDirectiveNoFileComp
+}
