@@ -13,7 +13,6 @@ import (
 
 	"git.sr.ht/~emersion/gqlclient"
 	"github.com/dustin/go-humanize"
-	"github.com/juju/ansiterm/tabwriter"
 	"github.com/spf13/cobra"
 
 	"git.sr.ht/~emersion/hut/srht/gitsrht"
@@ -55,13 +54,18 @@ func newGitCreateCommand() *cobra.Command {
 			importURLPtr = &importURL
 		}
 
+		var description *string
+		if desc != "" {
+			description = &desc
+		}
+
 		repo, err := gitsrht.CreateRepository(c.Client, ctx, args[0],
-			gitVisibility, desc, importURLPtr)
+			gitVisibility, description, importURLPtr)
 		if err != nil {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created repository %s\n", repo.Name)
+		log.Printf("Created repository %s\n", repo.Name)
 
 		if clone {
 			ver, err := gitsrht.SshSettings(c.Client, ctx)
@@ -181,7 +185,7 @@ func newGitDeleteCommand() *cobra.Command {
 		id := getRepoID(c, ctx, name, owner)
 
 		if !autoConfirm && !getConfirmation(fmt.Sprintf("Do you really want to delete the repo %s", name)) {
-			fmt.Println("Aborted")
+			log.Println("Aborted")
 			return
 		}
 		repo, err := gitsrht.DeleteRepository(c.Client, ctx, id)
@@ -189,7 +193,7 @@ func newGitDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted repository %s\n", repo.Name)
+		log.Printf("Deleted repository %s\n", repo.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -248,7 +252,7 @@ func newGitArtifactUploadCommand() *cobra.Command {
 				log.Fatal(err)
 			}
 
-			fmt.Printf("Uploaded %s\n", artifact.Filename)
+			log.Printf("Uploaded %s\n", artifact.Filename)
 		}
 	}
 
@@ -332,7 +336,7 @@ func newGitArtifactDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted artifact %s\n", artifact.Filename)
+		log.Printf("Deleted artifact %s\n", artifact.Filename)
 	}
 
 	cmd := &cobra.Command{
@@ -359,41 +363,50 @@ func newGitACLCommand() *cobra.Command {
 func newGitACLListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
-		var name, instance string
+		var name, owner, instance string
 		if len(args) > 0 {
-			// TODO: handle owner
-			name, _, instance = parseResourceName(args[0])
+			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, _, instance, err = getRepoName(ctx, cmd)
+			name, owner, instance, err = getRepoName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
 		}
 
 		c := createClientWithInstance("git", cmd, instance)
-
-		user, err := gitsrht.AclByRepoName(c.Client, ctx, name)
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatal("no such user")
-		} else if user.Repository == nil {
-			log.Fatalf("no such repository %q", name)
+		var (
+			cursor   *gitsrht.Cursor
+			user     *gitsrht.User
+			username string
+			err      error
+		)
+		if owner != "" {
+			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		tw := tabwriter.NewWriter(os.Stdout, 0, 2, 2, ' ', 0)
-		defer tw.Flush()
-		for _, acl := range user.Repository.Acls.Results {
-			var mode string
-			if acl.Mode != nil {
-				mode = string(*acl.Mode)
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = gitsrht.AclByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = gitsrht.AclByRepoName(c.Client, ctx, name, cursor)
 			}
 
-			created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
-			fmt.Fprintf(tw, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
-				acl.Entity.CanonicalName, mode, created)
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatal("no such user")
+			} else if user.Repository == nil {
+				log.Fatalf("no such repository %q", name)
+			}
+
+			for _, acl := range user.Repository.Acls.Results {
+				printGitACLEntry(p, &acl)
+			}
+
+			cursor = user.Repository.Acls.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -406,6 +419,17 @@ func newGitACLListCommand() *cobra.Command {
 	return cmd
 }
 
+func printGitACLEntry(w io.Writer, acl *gitsrht.ACL) {
+	var mode string
+	if acl.Mode != nil {
+		mode = string(*acl.Mode)
+	}
+
+	created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
+	fmt.Fprintf(w, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
+		acl.Entity.CanonicalName, mode, created)
+}
+
 func newGitACLUpdateCommand() *cobra.Command {
 	var mode string
 	run := func(cmd *cobra.Command, args []string) {
@@ -433,7 +457,7 @@ func newGitACLUpdateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Updated access rights for %s\n", acl.Entity.CanonicalName)
+		log.Printf("Updated access rights for %s\n", acl.Entity.CanonicalName)
 	}
 
 	cmd := &cobra.Command{
@@ -466,7 +490,7 @@ func newGitACLDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete ACL entry with ID %d", id)
 		}
 
-		fmt.Printf("Deleted ACL entry for %s in repository %s\n", acl.Entity.CanonicalName, acl.Repository.Name)
+		log.Printf("Deleted ACL entry for %s in repository %s\n", acl.Entity.CanonicalName, acl.Repository.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -602,7 +626,7 @@ func newGitUserWebhookCreateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created user webhook with ID %d\n", webhook.Id)
+		log.Printf("Created user webhook with ID %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -626,15 +650,21 @@ func newGitUserWebhookListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("git", cmd)
+		var cursor *gitsrht.Cursor
 
-		webhooks, err := gitsrht.UserWebhooks(c.Client, ctx)
-		if err != nil {
-			log.Fatal(err)
-		}
+		pagerify(func(p pager) bool {
+			webhooks, err := gitsrht.UserWebhooks(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
+			}
 
-		for _, webhook := range webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
+			for _, webhook := range webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = webhooks.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -661,7 +691,7 @@ func newGitUserWebhookDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted webhook %d\n", webhook.Id)
+		log.Printf("Deleted webhook %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -710,7 +740,12 @@ func newGitUpdateCommand() *cobra.Command {
 			input.HEAD = &branch
 		}
 
-		if readme != "" {
+		if readme == "" && cmd.Flags().Changed("readme") {
+			_, err := gitsrht.ClearCustomReadme(c.Client, ctx, id)
+			if err != nil {
+				log.Fatalf("failed to unset custom README: %v", err)
+			}
+		} else if readme != "" {
 			var (
 				b   []byte
 				err error
@@ -736,7 +771,7 @@ func newGitUpdateCommand() *cobra.Command {
 			log.Fatalf("failed to update repository %q", name)
 		}
 
-		fmt.Printf("Successfully updated repository %q\n", repo.Name)
+		log.Printf("Successfully updated repository %q\n", repo.Name)
 	}
 	cmd := &cobra.Command{
 		Use:               "update [repo]",
@@ -916,16 +951,23 @@ var completeAccessMode = cobra.FixedCompletions([]string{"RO", "RW"}, cobra.Shel
 
 func completeArtifacts(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 	ctx := cmd.Context()
-	repoName, _, instance, err := getRepoName(ctx, cmd)
+	repoName, owner, instance, err := getRepoName(ctx, cmd)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	c := createClientWithInstance("git", cmd, instance)
+	var user *gitsrht.User
 	var artifactList []string
 
-	user, err := gitsrht.ListArtifacts(c.Client, ctx, repoName)
-	if err != nil || user.Repository == nil {
+	if owner != "" {
+		username := strings.TrimLeft(owner, ownerPrefixes)
+		user, err = gitsrht.ListArtifactsByUser(c.Client, ctx, username, repoName)
+	} else {
+		user, err = gitsrht.ListArtifacts(c.Client, ctx, repoName)
+	}
+
+	if err != nil || user == nil || user.Repository == nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
@@ -956,7 +998,7 @@ func completeGitUserWebhookID(cmd *cobra.Command, args []string, toComplete stri
 	c := createClient("git", cmd)
 	var webhookList []string
 
-	webhooks, err := gitsrht.UserWebhooks(c.Client, ctx)
+	webhooks, err := gitsrht.UserWebhooks(c.Client, ctx, nil)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
@@ -991,3 +1033,26 @@ func completeBranches(cmd *cobra.Command, args []string, toComplete string) ([]s
 
 	return user.Repository.References.Heads(), cobra.ShellCompDirectiveNoFileComp
 }
+
+func completeCoMaintainers(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	ctx := cmd.Context()
+	// Since completeCoMaintainers is intended to be called from other services
+	// than git, we cannot use getRepoName which requires the "repo" flag to be set.
+	repoName, _, instace, err := guessGitRepoName(ctx, cmd)
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+	c := createClientWithInstance("git", cmd, instace)
+
+	var userList []string
+	user, err := gitsrht.CompleteCoMaintainers(c.Client, ctx, repoName)
+	if err != nil || user.Repositories == nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	for _, acl := range user.Repository.Acls.Results {
+		userList = append(userList, acl.Entity.CanonicalName)
+	}
+
+	return userList, cobra.ShellCompDirectiveNoFileComp
+}
