@@ -14,7 +14,6 @@ import (
 	"strings"
 
 	"github.com/dustin/go-humanize"
-	"github.com/juju/ansiterm/tabwriter"
 
 	"github.com/spf13/cobra"
 
@@ -37,6 +36,7 @@ func newListsCommand() *cobra.Command {
 	cmd.AddCommand(newListsACLCommand())
 	cmd.AddCommand(newListsUserWebhookCommand())
 	cmd.AddCommand(newListsWebhookCommand())
+	cmd.AddCommand(newListsSubscriptions())
 	cmd.PersistentFlags().StringP("mailing-list", "l", "", "mailing list name")
 	cmd.RegisterFlagCompletionFunc("mailing-list", completeList)
 	return cmd
@@ -49,7 +49,7 @@ func newListsDeleteCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -61,7 +61,7 @@ func newListsDeleteCommand() *cobra.Command {
 		id := getMailingListID(c, ctx, name, owner)
 
 		if !autoConfirm && !getConfirmation(fmt.Sprintf("Do you really want to delete the list %s", name)) {
-			fmt.Println("Aborted")
+			log.Println("Aborted")
 			return
 		}
 
@@ -72,7 +72,7 @@ func newListsDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete list with ID %d", id)
 		}
 
-		fmt.Printf("Deleted mailing list %s\n", list.Name)
+		log.Printf("Deleted mailing list %s\n", list.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -144,7 +144,7 @@ func newListsSubscribeCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -160,7 +160,7 @@ func newListsSubscribeCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Subscribed to %s/%s/%s\n", c.BaseURL, subscription.List.Owner.CanonicalName, subscription.List.Name)
+		log.Printf("Subscribed to %s/%s/%s\n", c.BaseURL, subscription.List.Owner.CanonicalName, subscription.List.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -179,7 +179,7 @@ func newListsUnsubscribeCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -197,7 +197,7 @@ func newListsUnsubscribeCommand() *cobra.Command {
 			log.Fatalf("you were not subscribed to %s/%s/%s", c.BaseURL, owner, name)
 		}
 
-		fmt.Printf("Unsubscribed from %s/%s/%s\n", c.BaseURL, subscription.List.Owner.CanonicalName, subscription.List.Name)
+		log.Printf("Unsubscribed from %s/%s/%s\n", c.BaseURL, subscription.List.Owner.CanonicalName, subscription.List.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -247,14 +247,14 @@ func newListsCreateCommand() *cobra.Command {
 			description = &text
 		}
 
-		tracker, err := listssrht.CreateMailingList(c.Client, ctx, args[0], description, listVisibility)
+		list, err := listssrht.CreateMailingList(c.Client, ctx, args[0], description, listVisibility)
 		if err != nil {
 			log.Fatal(err)
-		} else if tracker == nil {
+		} else if list == nil {
 			log.Fatal("failed to create mailing list")
 		}
 
-		fmt.Printf("Created mailing list %q\n", tracker.Name)
+		log.Printf("Created mailing list %q\n", list.Name)
 	}
 	cmd := &cobra.Command{
 		Use:               "create <name>",
@@ -276,7 +276,7 @@ func newListsArchiveCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -362,7 +362,7 @@ func newListsPatchsetListCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -373,70 +373,63 @@ func newListsPatchsetListCommand() *cobra.Command {
 		c := createClientWithInstance("lists", cmd, instance)
 
 		var (
-			err     error
-			patches *listssrht.PatchsetCursor
+			cursor   *listssrht.Cursor
+			patches  *listssrht.PatchsetCursor
+			user     *listssrht.User
+			username string
+			err      error
 		)
 
 		if byUser {
-			var user *listssrht.User
 			if len(args) > 0 {
-				name = strings.TrimLeft(name, ownerPrefixes)
-				user, err = listssrht.PatchesByUser(c.Client, ctx, name)
-			} else {
-				user, err = listssrht.Patches(c.Client, ctx)
+				username = strings.TrimLeft(name, ownerPrefixes)
 			}
-
-			if err != nil {
-				log.Fatal(err)
-			} else if user == nil {
-				log.Fatalf("no such user %q", name)
-			}
-			patches = user.Patches
 		} else {
-			var (
-				user     *listssrht.User
-				username string
-			)
-
 			if owner != "" {
 				username = strings.TrimLeft(owner, ownerPrefixes)
-				user, err = listssrht.ListPatchesByUser(c.Client, ctx, username, name)
-			} else {
-				user, err = listssrht.ListPatches(c.Client, ctx, name)
 			}
-
-			if err != nil {
-				log.Fatal(err)
-			} else if user == nil {
-				log.Fatalf("no such user %q", username)
-			} else if user.List == nil {
-				log.Fatalf("no such list %q", name)
-			}
-			patches = user.List.Patches
 		}
 
-		tw := tabwriter.NewWriter(os.Stdout, 0, 4, 2, ' ', 0)
-		defer tw.Flush()
-		for _, patchset := range patches.Results {
-			s := fmt.Sprintf("%s\t%s\t", termfmt.DarkYellow.Sprintf("#%d", patchset.Id), patchset.Status.TermString())
-			if patchset.Prefix != nil && *patchset.Prefix != "" {
-				s += fmt.Sprintf("[%s] ", *patchset.Prefix)
-			}
-			s += patchset.Subject
-			if patchset.Version != 1 {
-				s += fmt.Sprintf(" v%d", patchset.Version)
-			}
-
-			created := termfmt.Dim.String(humanize.Time(patchset.Created.Time))
-
+		pagerify(func(p pager) bool {
 			if byUser {
-				s += fmt.Sprintf("\t%s/%s\t%s", patchset.List.Owner.CanonicalName,
-					patchset.List.Name, created)
+				if username != "" {
+					user, err = listssrht.PatchesByUser(c.Client, ctx, name, cursor)
+				} else {
+					user, err = listssrht.Patches(c.Client, ctx, cursor)
+				}
+
+				if err != nil {
+					log.Fatal(err)
+				} else if user == nil {
+					log.Fatalf("no such user %q", name)
+				}
+
+				patches = user.Patches
 			} else {
-				s += fmt.Sprintf("\t%s\t%s", patchset.Submitter.CanonicalName, created)
+				if username != "" {
+					user, err = listssrht.ListPatchesByUser(c.Client, ctx, username, name, cursor)
+				} else {
+					user, err = listssrht.ListPatches(c.Client, ctx, name, cursor)
+				}
+
+				if err != nil {
+					log.Fatal(err)
+				} else if user == nil {
+					log.Fatalf("no such user %q", username)
+				} else if user.List == nil {
+					log.Fatalf("no such list %q", name)
+				}
+
+				patches = user.List.Patches
 			}
-			fmt.Fprintln(tw, s)
-		}
+
+			for _, patchset := range patches.Results {
+				printPatchset(p, byUser, &patchset)
+			}
+
+			cursor = patches.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -450,6 +443,27 @@ func newListsPatchsetListCommand() *cobra.Command {
 	return cmd
 }
 
+func printPatchset(w io.Writer, byUser bool, patchset *listssrht.Patchset) {
+	s := fmt.Sprintf("%s\t%s\t", termfmt.DarkYellow.Sprintf("#%d", patchset.Id), patchset.Status.TermString())
+	if patchset.Prefix != nil && *patchset.Prefix != "" {
+		s += fmt.Sprintf("[%s] ", *patchset.Prefix)
+	}
+	s += patchset.Subject
+	if patchset.Version != 1 {
+		s += fmt.Sprintf(" v%d", patchset.Version)
+	}
+
+	created := termfmt.Dim.String(humanize.Time(patchset.Created.Time))
+
+	if byUser {
+		s += fmt.Sprintf("\t%s/%s\t%s", patchset.List.Owner.CanonicalName,
+			patchset.List.Name, created)
+	} else {
+		s += fmt.Sprintf("\t%s\t%s", patchset.Submitter.CanonicalName, created)
+	}
+	fmt.Fprintln(w, s)
+}
+
 func newListsPatchsetUpdateCommand() *cobra.Command {
 	var status string
 	run := func(cmd *cobra.Command, args []string) {
@@ -473,7 +487,7 @@ func newListsPatchsetUpdateCommand() *cobra.Command {
 			log.Fatalf("failed to update patchset with ID %d", id)
 		}
 
-		fmt.Printf("Updated patchset %q by %s\n", patch.Subject, patch.Submitter.CanonicalName)
+		log.Printf("Updated patchset %q by %s\n", patch.Subject, patch.Submitter.CanonicalName)
 	}
 
 	cmd := &cobra.Command{
@@ -504,16 +518,24 @@ func newListsPatchsetShowCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 		c := createClientWithInstance("lists", cmd, instance)
+		var cursor *listssrht.Cursor
 
-		patchset, err := listssrht.PatchsetById(c.Client, ctx, id)
-		if err != nil {
-			log.Fatal(err)
-		} else if patchset == nil {
-			log.Fatalf("no such patchset %d", id)
-		}
+		for {
+			patchset, err := listssrht.PatchsetById(c.Client, ctx, id, cursor)
+			if err != nil {
+				log.Fatal(err)
+			} else if patchset == nil {
+				log.Fatalf("no such patchset %d", id)
+			}
 
-		for _, patch := range patchset.Patches.Results {
-			formatPatch(os.Stdout, &patch)
+			for _, patch := range patchset.Patches.Results {
+				formatPatch(os.Stdout, &patch)
+			}
+
+			cursor = patchset.Patches.Cursor
+			if cursor == nil {
+				break
+			}
 		}
 	}
 	return &cmd
@@ -528,17 +550,25 @@ func newListsPatchsetApplyCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 		c := createClientWithInstance("lists", cmd, instance)
-
-		patchset, err := listssrht.PatchsetById(c.Client, ctx, id)
-		if err != nil {
-			log.Fatal(err)
-		} else if patchset == nil {
-			log.Fatalf("no such patchset %d", id)
-		}
-
+		var cursor *listssrht.Cursor
 		var mbox bytes.Buffer
-		for _, patch := range patchset.Patches.Results {
-			formatPatch(&mbox, &patch)
+
+		for {
+			patchset, err := listssrht.PatchsetById(c.Client, ctx, id, cursor)
+			if err != nil {
+				log.Fatal(err)
+			} else if patchset == nil {
+				log.Fatalf("no such patchset %d", id)
+			}
+
+			for _, patch := range patchset.Patches.Results {
+				formatPatch(&mbox, &patch)
+			}
+
+			cursor = patchset.Patches.Cursor
+			if cursor == nil {
+				break
+			}
 		}
 
 		applyCmd := exec.Command("git", "am", "-3")
@@ -583,56 +613,81 @@ func newListsACLListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 
-		var name, instance string
+		var name, owner, instance string
 		if len(args) > 0 {
-			// TODO: handle owner
-			name, _, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
-			name, _, instance, err = getMailingListName(ctx, cmd)
+			name, owner, instance, err = getMailingListName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
 		}
 
 		c := createClientWithInstance("lists", cmd, instance)
-
-		user, err := listssrht.AclByListName(c.Client, ctx, name)
-		if err != nil {
-			log.Fatal(err)
-		} else if user.List == nil {
-			log.Fatalf("no such list %q", name)
+		var (
+			cursor   *listssrht.Cursor
+			user     *listssrht.User
+			username string
+			err      error
+		)
+		if owner != "" {
+			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		fmt.Println(termfmt.Bold.Sprint("Default permissions"))
-		fmt.Println(user.List.DefaultACL.TermString())
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = listssrht.AclByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = listssrht.AclByListName(c.Client, ctx, name, cursor)
+			}
 
-		if len(user.List.Acl.Results) > 0 {
-			fmt.Println(termfmt.Bold.Sprint("\nUser permissions"))
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.List == nil {
+				log.Fatalf("no such list %q", name)
+			}
 
-		tw := tabwriter.NewWriter(os.Stdout, 0, 2, 2, ' ', 0)
-		defer tw.Flush()
-		for _, acl := range user.List.Acl.Results {
-			s := fmt.Sprintf("%s browse  %s reply  %s post  %s moderate",
-				listssrht.PermissionIcon(acl.Browse), listssrht.PermissionIcon(acl.Reply),
-				listssrht.PermissionIcon(acl.Post), listssrht.PermissionIcon(acl.Moderate))
-			created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
-			fmt.Fprintf(tw, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
-				acl.Entity.CanonicalName, s, created)
-		}
+			if cursor == nil {
+				// only print once
+				fmt.Fprintln(p, termfmt.Bold.Sprint("Default permissions"))
+				fmt.Fprintln(p, user.List.DefaultACL.TermString())
+
+				if len(user.List.Acl.Results) > 0 {
+					fmt.Fprintln(p, termfmt.Bold.Sprint("\nUser permissions"))
+				}
+			}
+
+			for _, acl := range user.List.Acl.Results {
+				printListsACLEntry(p, &acl)
+			}
+
+			cursor = user.List.Acl.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
 		Use:               "list",
 		Short:             "List ACL entries",
 		Args:              cobra.MaximumNArgs(1),
-		ValidArgsFunction: cobra.NoFileCompletions,
+		ValidArgsFunction: completeList,
 		Run:               run,
 	}
 	return cmd
 }
 
+func printListsACLEntry(w io.Writer, acl *listssrht.MailingListACL) {
+	s := fmt.Sprintf("%s browse  %s reply  %s post  %s moderate",
+		listssrht.PermissionIcon(acl.Browse), listssrht.PermissionIcon(acl.Reply),
+		listssrht.PermissionIcon(acl.Post), listssrht.PermissionIcon(acl.Moderate))
+	created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
+	fmt.Fprintf(w, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
+		acl.Entity.CanonicalName, s, created)
+}
+
 func newListsACLDeleteCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
@@ -650,7 +705,7 @@ func newListsACLDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete ACL entry with ID %d", id)
 		}
 
-		fmt.Printf("Deleted ACL entry for %q in mailing list %q\n", acl.Entity.CanonicalName, acl.List.Name)
+		log.Printf("Deleted ACL entry for %q in mailing list %q\n", acl.Entity.CanonicalName, acl.List.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -697,7 +752,7 @@ func newListsUserWebhookCreateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created user webhook with ID %d\n", webhook.Id)
+		log.Printf("Created user webhook with ID %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -721,15 +776,21 @@ func newListsUserWebhookListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("lists", cmd)
+		var cursor *listssrht.Cursor
 
-		webhooks, err := listssrht.UserWebhooks(c.Client, ctx)
-		if err != nil {
-			log.Fatal(err)
-		}
+		pagerify(func(p pager) bool {
+			webhooks, err := listssrht.UserWebhooks(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
+			}
 
-		for _, webhook := range webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
+			for _, webhook := range webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = webhooks.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -756,7 +817,7 @@ func newListsUserWebhookDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted webhook %d\n", webhook.Id)
+		log.Printf("Deleted webhook %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -789,7 +850,7 @@ func newListsWebhookCreateCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -842,7 +903,7 @@ func newListsWebhookListCommand() *cobra.Command {
 
 		var name, owner, instance string
 		if len(args) > 0 {
-			name, owner, instance = parseResourceName(args[0])
+			name, owner, instance = parseMailingListName(args[0])
 		} else {
 			var err error
 			name, owner, instance, err = getMailingListName(ctx, cmd)
@@ -854,29 +915,37 @@ func newListsWebhookListCommand() *cobra.Command {
 		c := createClientWithInstance("lists", cmd, instance)
 
 		var (
+			cursor   *listssrht.Cursor
 			user     *listssrht.User
 			username string
 			err      error
 		)
-
 		if owner != "" {
 			username = strings.TrimLeft(owner, ownerPrefixes)
-			user, err = listssrht.MailingListWebhooksByUser(c.Client, ctx, username, name)
-		} else {
-			user, err = listssrht.MailingListWebhooks(c.Client, ctx, name)
 		}
 
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatalf("no such user %q", username)
-		} else if user.List == nil {
-			log.Fatalf("no such tracker %q", name)
-		}
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = listssrht.MailingListWebhooksByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = listssrht.MailingListWebhooks(c.Client, ctx, name, cursor)
+			}
 
-		for _, webhook := range user.List.Webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.List == nil {
+				log.Fatalf("no such mailing list %q", name)
+			}
+
+			for _, webhook := range user.List.Webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = user.List.Webhooks.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -917,6 +986,61 @@ func newListsWebhookDeleteCommand() *cobra.Command {
 	return cmd
 }
 
+func newListsSubscriptions() *cobra.Command {
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		c := createClient("lists", cmd)
+		var cursor *listssrht.Cursor
+
+		pagerify(func(p pager) bool {
+			subscriptions, err := listssrht.Subscriptions(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
+			}
+
+			for _, sub := range subscriptions.Results {
+				printMailingListSubscription(p, &sub)
+			}
+
+			cursor = subscriptions.Cursor
+			return cursor == nil
+		})
+	}
+
+	cmd := &cobra.Command{
+		Use:   "subscriptions",
+		Short: "List mailing list subscriptions",
+		Args:  cobra.ExactArgs(0),
+		Run:   run,
+	}
+	return cmd
+}
+
+func printMailingListSubscription(w io.Writer, sub *listssrht.ActivitySubscription) {
+	mlSub, ok := sub.Value.(*listssrht.MailingListSubscription)
+	if !ok {
+		return
+	}
+
+	created := termfmt.Dim.String(humanize.Time(sub.Created.Time))
+	fmt.Fprintf(w, "%s/%s %s\n", mlSub.List.Owner.CanonicalName, mlSub.List.Name, created)
+}
+
+// parseMailingListName parses a mailing list name, following either the
+// generic resource name syntax, or "<owner>/<name>@<instance>".
+func parseMailingListName(s string) (name, owner, instance string) {
+	slash := strings.Index(s, "/")
+	at := strings.Index(s, "@")
+	if strings.Count(s, "/") != 1 || strings.Count(s, "@") != 1 || at < slash {
+		return parseResourceName(s)
+	}
+
+	owner = s[:slash]
+	name = s[slash+1 : at]
+	instance = s[at+1:]
+	return name, owner, instance
+}
+
 func getMailingListID(c *Client, ctx context.Context, name, owner string) int32 {
 	var (
 		user     *listssrht.User
@@ -948,7 +1072,7 @@ func getMailingListName(ctx context.Context, cmd *cobra.Command) (name, owner, i
 	if err != nil {
 		return "", "", "", err
 	} else if s != "" {
-		name, owner, instance = parseResourceName(s)
+		name, owner, instance = parseMailingListName(s)
 		return name, owner, instance, nil
 	}
 
@@ -968,13 +1092,7 @@ func guessMailingListName(ctx context.Context) (name, owner, instance string, er
 		return "", "", "", errors.New("no mailing list specified and no mailing list configured for current Git repository")
 	}
 
-	parts := strings.SplitN(addr.Address, "@", 2)
-	if len(parts) != 2 {
-		return "", "", "", fmt.Errorf("invalid email address %q", addr.Address)
-	}
-
-	name, owner, _ = parseResourceName(parts[0])
-	instance = parts[1]
+	name, owner, instance = parseMailingListName(addr.Address)
 	return name, owner, instance, nil
 }
 
@@ -1057,6 +1175,10 @@ func completePatchsetID(cmd *cobra.Command, args []string, toComplete string) ([
 			continue
 		}
 
+		if cmd.Name() == "update" && strings.EqualFold(cmd.Flag("status").Value.String(), string(patchset.Status)) {
+			continue
+		}
+
 		s := fmt.Sprintf("%d\t", patchset.Id)
 		if patchset.Prefix != nil && *patchset.Prefix != "" {
 			s += fmt.Sprintf("[%s] ", *patchset.Prefix)
@@ -1097,7 +1219,7 @@ func completeListsUserWebhookID(cmd *cobra.Command, args []string, toComplete st
 	c := createClient("lists", cmd)
 	var webhookList []string
 
-	webhooks, err := listssrht.UserWebhooks(c.Client, ctx)
+	webhooks, err := listssrht.UserWebhooks(c.Client, ctx, nil)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
