@@ -14,7 +14,6 @@ import (
 	"git.sr.ht/~emersion/hut/srht/todosrht"
 	"git.sr.ht/~emersion/hut/termfmt"
 	"github.com/dustin/go-humanize"
-	"github.com/juju/ansiterm/tabwriter"
 	"github.com/spf13/cobra"
 )
 
@@ -101,7 +100,7 @@ func newTodoDeleteCommand() *cobra.Command {
 		id := getTrackerID(c, ctx, name, owner)
 
 		if !autoConfirm && !getConfirmation(fmt.Sprintf("Do you really want to delete the tracker %s", name)) {
-			fmt.Println("Aborted")
+			log.Println("Aborted")
 			return
 		}
 
@@ -112,7 +111,7 @@ func newTodoDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete tracker %q", name)
 		}
 
-		fmt.Printf("Deleted tracker %s\n", tracker.Name)
+		log.Printf("Deleted tracker %s\n", tracker.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -150,7 +149,7 @@ func newTodoSubscribeCommand() *cobra.Command {
 			log.Fatalf("failed to subscribe to tracker %q", name)
 		}
 
-		fmt.Printf("Subscribed to %s/%s/%s\n", c.BaseURL, subscription.Tracker.Owner.CanonicalName, subscription.Tracker.Name)
+		log.Printf("Subscribed to %s/%s/%s\n", c.BaseURL, subscription.Tracker.Owner.CanonicalName, subscription.Tracker.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -187,7 +186,7 @@ func newTodoUnsubscribeCommand() *cobra.Command {
 			log.Fatalf("you were not subscribed to %s/%s/%s", c.BaseURL, owner, name)
 		}
 
-		fmt.Printf("Unsubscribed from %s/%s/%s\n", c.BaseURL, subscription.Tracker.Owner.CanonicalName, subscription.Tracker.Name)
+		log.Printf("Unsubscribed from %s/%s/%s\n", c.BaseURL, subscription.Tracker.Owner.CanonicalName, subscription.Tracker.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -244,7 +243,7 @@ func newTodoCreateCommand() *cobra.Command {
 			log.Fatal("failed to create tracker")
 		}
 
-		fmt.Printf("Created tracker %q\n", tracker.Name)
+		log.Printf("Created tracker %q\n", tracker.Name)
 	}
 	cmd := &cobra.Command{
 		Use:               "create <name>",
@@ -291,45 +290,37 @@ func newTodoTicketListCommand() *cobra.Command {
 
 		c := createClientWithInstance("todo", cmd, instance)
 		var (
+			cursor   *todosrht.Cursor
 			user     *todosrht.User
 			username string
 		)
-
 		if owner != "" {
 			username = strings.TrimLeft(owner, ownerPrefixes)
-			user, err = todosrht.TicketsByUser(c.Client, ctx, username, name)
-		} else {
-			user, err = todosrht.Tickets(c.Client, ctx, name)
 		}
 
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatalf("no such user %q", username)
-		} else if user.Tracker == nil {
-			log.Fatalf("no such tracker %q", name)
-		}
-
-		for _, ticket := range user.Tracker.Tickets.Results {
-			var labels string
-			s := termfmt.DarkYellow.Sprintf("#%d %s ", ticket.Id, ticket.Status.TermString())
-			if ticket.Status == todosrht.TicketStatusResolved && ticket.Resolution != todosrht.TicketResolutionClosed {
-				s += termfmt.Green.Sprintf("%s ", strings.ToLower(string(ticket.Resolution)))
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = todosrht.TicketsByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = todosrht.Tickets(c.Client, ctx, name, cursor)
 			}
 
-			if len(ticket.Labels) > 0 {
-				labels = " "
-				for i, label := range ticket.Labels {
-					labels += label.TermString()
-					if i != len(ticket.Labels)-1 {
-						labels += " "
-					}
-				}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.Tracker == nil {
+				log.Fatalf("no such tracker %q", name)
 			}
-			s += fmt.Sprintf("%s%s (%s %s)", ticket.Subject, labels,
-				ticket.Submitter.CanonicalName, humanize.Time(ticket.Created.Time))
-			fmt.Println(s)
-		}
+
+			for _, ticket := range user.Tracker.Tickets.Results {
+				printTicket(p, &ticket)
+			}
+
+			cursor = user.Tracker.Tickets.Cursor
+			return cursor == nil
+		})
+
 	}
 
 	cmd := &cobra.Command{
@@ -341,6 +332,27 @@ func newTodoTicketListCommand() *cobra.Command {
 	return cmd
 }
 
+func printTicket(w io.Writer, ticket *todosrht.Ticket) {
+	var labels string
+	s := termfmt.DarkYellow.Sprintf("#%d %s ", ticket.Id, ticket.Status.TermString())
+	if ticket.Status == todosrht.TicketStatusResolved && ticket.Resolution != todosrht.TicketResolutionClosed {
+		s += termfmt.Green.Sprintf("%s ", strings.ToLower(string(ticket.Resolution)))
+	}
+
+	if len(ticket.Labels) > 0 {
+		labels = " "
+		for i, label := range ticket.Labels {
+			labels += label.TermString()
+			if i != len(ticket.Labels)-1 {
+				labels += " "
+			}
+		}
+	}
+	s += fmt.Sprintf("%s%s (%s %s)", ticket.Subject, labels,
+		ticket.Submitter.CanonicalName, humanize.Time(ticket.Created.Time))
+	fmt.Fprintln(w, s)
+}
+
 func newTodoTicketCommentCommand() *cobra.Command {
 	var stdin bool
 	var status, resolution string
@@ -402,7 +414,7 @@ func newTodoTicketCommentCommand() *cobra.Command {
 		}
 
 		if strings.TrimSpace(input.Text) == "" {
-			fmt.Println("Aborted writing empty comment")
+			log.Println("Aborted writing empty comment")
 			return
 		}
 
@@ -413,7 +425,7 @@ func newTodoTicketCommentCommand() *cobra.Command {
 			log.Fatalf("failed to comment on ticket with ID %d", ticketID)
 		}
 
-		fmt.Printf("Commented on %s\n", event.Ticket.Subject)
+		log.Printf("Commented on %s\n", event.Ticket.Subject)
 	}
 
 	cmd := &cobra.Command{
@@ -479,7 +491,7 @@ func newTodoTicketStatusCommand() *cobra.Command {
 			log.Fatalf("failed to update status of ticket with ID %d", ticketID)
 		}
 
-		fmt.Printf("Updated status of %s\n", event.Ticket.Subject)
+		log.Printf("Updated status of %s\n", event.Ticket.Subject)
 	}
 
 	cmd := &cobra.Command{
@@ -515,7 +527,7 @@ func newTodoTicketSubscribeCommand() *cobra.Command {
 			log.Fatalf("failed to subscribe to ticket %d", ticketID)
 		}
 
-		fmt.Printf("Subscribed to %s/%s/%s/%d\n", c.BaseURL, owner, name, subscription.Ticket.Id)
+		log.Printf("Subscribed to %s/%s/%s/%d\n", c.BaseURL, owner, name, subscription.Ticket.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -547,7 +559,7 @@ func newTodoTicketUnsubscribeCommand() *cobra.Command {
 			log.Fatalf("you were not subscribed to ticket with ID %d", ticketID)
 		}
 
-		fmt.Printf("Unsubscribed from %s/%s/%s/%d\n", c.BaseURL, owner, name, subscription.Ticket.Id)
+		log.Printf("Unsubscribed from %s/%s/%s/%d\n", c.BaseURL, owner, name, subscription.Ticket.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -587,7 +599,7 @@ func newTodoTicketAssignCommand() *cobra.Command {
 			log.Fatal("failed to assign user")
 		}
 
-		fmt.Printf("Assigned %q to %q\n", userName, event.Ticket.Subject)
+		log.Printf("Assigned %q to %q\n", userName, event.Ticket.Subject)
 	}
 
 	cmd := &cobra.Command{
@@ -630,7 +642,7 @@ func newTodoTicketUnassignCommand() *cobra.Command {
 			log.Fatal("failed to unassign user")
 		}
 
-		fmt.Printf("Unassigned %q from %q\n", userName, event.Ticket.Subject)
+		log.Printf("Unassigned %q from %q\n", userName, event.Ticket.Subject)
 	}
 
 	cmd := &cobra.Command{
@@ -660,7 +672,7 @@ func newTodoTicketDeleteCommand() *cobra.Command {
 		trackerID := getTrackerID(c, ctx, name, owner)
 
 		if !autoConfirm && !getConfirmation(fmt.Sprintf("Do you really want to delete the ticket with ID %d", ticketID)) {
-			fmt.Println("Aborted")
+			log.Println("Aborted")
 			return
 		}
 
@@ -671,7 +683,7 @@ func newTodoTicketDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete ticket %d", ticketID)
 		}
 
-		fmt.Printf("Deleted ticket %q\n", ticket.Subject)
+		log.Printf("Deleted ticket %q\n", ticket.Subject)
 	}
 
 	cmd := &cobra.Command{
@@ -754,8 +766,21 @@ func newTodoTicketShowCommand() *cobra.Command {
 			fmt.Println(*ticket.Body)
 		}
 
-		// TODO: handle events
-		// Depends on https://todo.sr.ht/~emersion/gqlclient/3
+		for i := len(ticket.Events.Results) - 1; i >= 0; i-- {
+			event := ticket.Events.Results[i]
+			for _, change := range event.Changes {
+				comment, ok := change.Value.(*todosrht.Comment)
+				if !ok {
+					continue
+				}
+				author := termfmt.Bold.String(comment.Author.CanonicalName)
+				created := termfmt.Dim.String("(" + humanize.Time(event.Created.Time) + ")")
+				fmt.Println()
+				fmt.Printf("%v %v\n", author, created)
+				fmt.Print(indent(comment.Text, "  "))
+				fmt.Println()
+			}
+		}
 	}
 	cmd := &cobra.Command{
 		Use:               "show <ID>",
@@ -808,7 +833,7 @@ func newTodoTicketWebhookCreateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created ticket webhook with ID %d\n", webhook.Id)
+		log.Printf("Created ticket webhook with ID %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -839,28 +864,36 @@ func newTodoTicketWebhookListCommand() *cobra.Command {
 
 		c := createClientWithInstance("todo", cmd, instance)
 		var (
+			cursor   *todosrht.Cursor
 			user     *todosrht.User
 			username string
 		)
-
 		if owner != "" {
 			username = strings.TrimLeft(owner, ownerPrefixes)
-			user, err = todosrht.TicketWebhooksByUser(c.Client, ctx, username, name, ticketID)
-		} else {
-			user, err = todosrht.TicketWebhooks(c.Client, ctx, name, ticketID)
 		}
 
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatalf("no such user %q", username)
-		} else if user.Tracker == nil {
-			log.Fatalf("no such tracker %q", name)
-		}
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = todosrht.TicketWebhooksByUser(c.Client, ctx, username, name, ticketID, cursor)
+			} else {
+				user, err = todosrht.TicketWebhooks(c.Client, ctx, name, ticketID, cursor)
+			}
 
-		for _, webhook := range user.Tracker.Ticket.Webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.Tracker == nil {
+				log.Fatalf("no such tracker %q", name)
+			}
+
+			for _, webhook := range user.Tracker.Ticket.Webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = user.Tracker.Ticket.Webhooks.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -888,7 +921,7 @@ func newTodoTicketWebhookDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted webhook %d\n", webhook.Id)
+		log.Printf("Deleted webhook %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -922,28 +955,36 @@ func newTodoLabelListCommand() *cobra.Command {
 
 		c := createClientWithInstance("todo", cmd, instance)
 		var (
+			cursor   *todosrht.Cursor
 			user     *todosrht.User
 			username string
 		)
-
 		if owner != "" {
 			username = strings.TrimLeft(owner, ownerPrefixes)
-			user, err = todosrht.LabelsByUser(c.Client, ctx, username, name)
-		} else {
-			user, err = todosrht.Labels(c.Client, ctx, name)
 		}
 
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatalf("no such user %q", username)
-		} else if user.Tracker == nil {
-			log.Fatalf("no such tracker %q", name)
-		}
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = todosrht.LabelsByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = todosrht.Labels(c.Client, ctx, name, cursor)
+			}
 
-		for _, label := range user.Tracker.Labels.Results {
-			fmt.Println(label.TermString())
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.Tracker == nil {
+				log.Fatalf("no such tracker %q", name)
+			}
+
+			for _, label := range user.Tracker.Labels.Results {
+				fmt.Fprintln(p, label.TermString())
+			}
+
+			cursor = user.Tracker.Labels.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -974,7 +1015,7 @@ func newTodoLabelDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted label %s\n", label.Name)
+		log.Printf("Deleted label %s\n", label.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -1010,7 +1051,7 @@ func newTodoLabelCreateCommand() *cobra.Command {
 			log.Fatal("failed to create label")
 		}
 
-		fmt.Printf("Created label %s\n", label.TermString())
+		log.Printf("Created label %s\n", label.TermString())
 	}
 
 	cmd := &cobra.Command{
@@ -1041,45 +1082,60 @@ func newTodoACLCommand() *cobra.Command {
 func newTodoACLListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
-		var name, instance string
+		var name, owner, instance string
 		if len(args) > 0 {
-			// TODO: handle owner
-			name, _, instance = parseResourceName(args[0])
+			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, _, instance, err = getTrackerName(ctx, cmd)
+			name, owner, instance, err = getTrackerName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
 		}
 
 		c := createClientWithInstance("todo", cmd, instance)
-
-		user, err := todosrht.AclByTrackerName(c.Client, ctx, name)
-		if err != nil {
-			log.Fatal(err)
-		} else if user.Tracker == nil {
-			log.Fatalf("no such tracker %q", name)
+		var (
+			cursor   *todosrht.Cursor
+			user     *todosrht.User
+			username string
+			err      error
+		)
+		if owner != "" {
+			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		fmt.Println(termfmt.Bold.Sprint("Default permissions"))
-		fmt.Println(user.Tracker.DefaultACL.TermString())
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = todosrht.AclByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = todosrht.AclByTrackerName(c.Client, ctx, name, cursor)
+			}
 
-		if len(user.Tracker.Acls.Results) > 0 {
-			fmt.Println(termfmt.Bold.Sprint("\nUser permissions"))
-		}
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.Tracker == nil {
+				log.Fatalf("no such tracker %q", name)
+			}
 
-		tw := tabwriter.NewWriter(os.Stdout, 0, 2, 2, ' ', 0)
-		defer tw.Flush()
-		for _, acl := range user.Tracker.Acls.Results {
-			s := fmt.Sprintf("%s browse  %s submit  %s comment  %s edit  %s triage",
-				todosrht.PermissionIcon(acl.Browse), todosrht.PermissionIcon(acl.Submit),
-				todosrht.PermissionIcon(acl.Comment), todosrht.PermissionIcon(acl.Edit),
-				todosrht.PermissionIcon(acl.Triage))
-			created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
-			fmt.Fprintf(tw, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
-				acl.Entity.CanonicalName, s, created)
-		}
+			if cursor == nil {
+				// only print once
+				fmt.Fprintln(p, termfmt.Bold.Sprint("Default permissions"))
+				fmt.Fprintln(p, user.Tracker.DefaultACL.TermString())
+
+				if len(user.Tracker.Acls.Results) > 0 {
+					fmt.Fprintln(p, termfmt.Bold.Sprint("\nUser permissions"))
+				}
+			}
+
+			for _, acl := range user.Tracker.Acls.Results {
+				printACLEntry(p, &acl)
+			}
+
+			cursor = user.Tracker.Acls.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -1092,6 +1148,16 @@ func newTodoACLListCommand() *cobra.Command {
 	return cmd
 }
 
+func printACLEntry(w io.Writer, acl *todosrht.TrackerACL) {
+	s := fmt.Sprintf("%s browse  %s submit  %s comment  %s edit  %s triage",
+		todosrht.PermissionIcon(acl.Browse), todosrht.PermissionIcon(acl.Submit),
+		todosrht.PermissionIcon(acl.Comment), todosrht.PermissionIcon(acl.Edit),
+		todosrht.PermissionIcon(acl.Triage))
+	created := termfmt.Dim.String(humanize.Time(acl.Created.Time))
+	fmt.Fprintf(w, "%s\t%s\t%s\t%s\n", termfmt.DarkYellow.Sprintf("#%d", acl.Id),
+		acl.Entity.CanonicalName, s, created)
+}
+
 func newTodoACLDeleteCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
@@ -1109,7 +1175,7 @@ func newTodoACLDeleteCommand() *cobra.Command {
 			log.Fatalf("failed to delete ACL entry with ID %d", id)
 		}
 
-		fmt.Printf("Deleted ACL entry for %q in tracker %q\n", acl.Entity.CanonicalName, acl.Tracker.Name)
+		log.Printf("Deleted ACL entry for %q in tracker %q\n", acl.Entity.CanonicalName, acl.Tracker.Name)
 	}
 
 	cmd := &cobra.Command{
@@ -1169,7 +1235,7 @@ func newTodoWebhookCreateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created tracker webhook with ID %d\n", webhook.Id)
+		log.Printf("Created tracker webhook with ID %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -1207,29 +1273,38 @@ func newTodoWebhookListCommand() *cobra.Command {
 		c := createClientWithInstance("todo", cmd, instance)
 
 		var (
+			cursor   *todosrht.Cursor
 			user     *todosrht.User
 			username string
 			err      error
 		)
-
 		if owner != "" {
 			username = strings.TrimLeft(owner, ownerPrefixes)
-			user, err = todosrht.TrackerWebhooksByUser(c.Client, ctx, username, name)
-		} else {
-			user, err = todosrht.TrackerWebhooks(c.Client, ctx, name)
 		}
 
-		if err != nil {
-			log.Fatal(err)
-		} else if user == nil {
-			log.Fatalf("no such user %q", username)
-		} else if user.Tracker == nil {
-			log.Fatalf("no such tracker %q", name)
-		}
+		pagerify(func(p pager) bool {
+			if username != "" {
+				user, err = todosrht.TrackerWebhooksByUser(c.Client, ctx, username, name, cursor)
+			} else {
+				user, err = todosrht.TrackerWebhooks(c.Client, ctx, name, cursor)
+			}
+
+			if err != nil {
+				log.Fatal(err)
+			} else if user == nil {
+				log.Fatalf("no such user %q", username)
+			} else if user.Tracker == nil {
+				log.Fatalf("no such tracker %q", name)
+			}
+
+			for _, webhook := range user.Tracker.Webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = user.Tracker.Webhooks.Cursor
+			return cursor == nil
+		})
 
-		for _, webhook := range user.Tracker.Webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
 	}
 
 	cmd := &cobra.Command{
@@ -1257,7 +1332,7 @@ func newTodoWebhookDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted webhook %d\n", webhook.Id)
+		log.Printf("Deleted webhook %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -1304,7 +1379,7 @@ func newTodoUserWebhookCreateCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Created user webhook with ID %d\n", webhook.Id)
+		log.Printf("Created user webhook with ID %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -1328,15 +1403,21 @@ func newTodoUserWebhookListCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
 		c := createClient("todo", cmd)
+		var cursor *todosrht.Cursor
 
-		webhooks, err := todosrht.UserWebhooks(c.Client, ctx)
-		if err != nil {
-			log.Fatal(err)
-		}
+		pagerify(func(p pager) bool {
+			webhooks, err := todosrht.UserWebhooks(c.Client, ctx, cursor)
+			if err != nil {
+				log.Fatal(err)
+			}
 
-		for _, webhook := range webhooks.Results {
-			fmt.Printf("%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
-		}
+			for _, webhook := range webhooks.Results {
+				fmt.Fprintf(p, "%s %s\n", termfmt.DarkYellow.Sprintf("#%d", webhook.Id), webhook.Url)
+			}
+
+			cursor = webhooks.Cursor
+			return cursor == nil
+		})
 	}
 
 	cmd := &cobra.Command{
@@ -1363,7 +1444,7 @@ func newTodoUserWebhookDeleteCommand() *cobra.Command {
 			log.Fatal(err)
 		}
 
-		fmt.Printf("Deleted webhook %d\n", webhook.Id)
+		log.Printf("Deleted webhook %d\n", webhook.Id)
 	}
 
 	cmd := &cobra.Command{
@@ -1435,7 +1516,7 @@ func newTodoTicketCreateCommand() *cobra.Command {
 		}
 
 		if input.Subject == "" {
-			fmt.Println("Aborting due to empty subject.")
+			log.Println("Aborting due to empty subject.")
 			os.Exit(1)
 		}
 
@@ -1446,7 +1527,7 @@ func newTodoTicketCreateCommand() *cobra.Command {
 			log.Fatal("failed to create ticket")
 		}
 
-		fmt.Printf("Created new ticket %v\n", termfmt.DarkYellow.Sprintf("#%v", ticket.Id))
+		log.Printf("Created new ticket %v\n", termfmt.DarkYellow.Sprintf("#%v", ticket.Id))
 	}
 
 	cmd := &cobra.Command{
@@ -1894,7 +1975,7 @@ func completeTodoUserWebhookID(cmd *cobra.Command, args []string, toComplete str
 	c := createClient("todo", cmd)
 	var webhookList []string
 
-	webhooks, err := todosrht.UserWebhooks(c.Client, ctx)
+	webhooks, err := todosrht.UserWebhooks(c.Client, ctx, nil)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
